import os
import string
from tempfile import TemporaryFile
from CGIHTTPServer import CGIHTTPRequestHandler

class CGIServer:
    def __init__(self,server_name,server_port):
        self.server_name = server_name
        self.server_port = server_port

    def handle_error(self, request, client_address):
        print '-'*40
        print 'Exception happened during processing of request from',
        print client_address
        import traceback
        traceback.print_exc() # XXX But this goes to stderr!
        print '-'*40

class CGIHTTPError(EnvironmentError):
    pass

class CGIHTTP(CGIHTTPRequestHandler):
    # 
    # Please modify to restrict the executable programs that
    # can be used (directories are relative to INSTANCE_HOME)
    # 
    # /cgi-bin nees to be last (i.e. longest match first)
    #
    cgi_directories = ['/cgi-bin/prd/jcn/unkei/solaris/zagent',
                       '/cgi-bin/prd/jcn/unkei/solaris',
                       '/cgi-bin/tst/jcn/unkei/solaris/zagent',
                       '/cgi-bin/tst/jcn/unkei/solaris',
                       '/cgi-bin/dev/jcn/unkei/solaris/zagent',
                       '/cgi-bin/dev/jcn/unkei/solaris',
                       '/cgi-bin']

    rbufsize = -1
    wbufsize = -1

    # disable/enable for debugging purposes only
    #have_fork = 0
    #have_popen2 = 0

    def __init__(self,method,url,headers,content,server_name,server_port):
        # patch for decoded args
        while url[-1:]=='=': url=url[:-1]
        # raw_requestline
        self.raw_requestline = '%s %s HTTP/1.0' % (method,url)
        # r/w temporary files
        self.rfile = TemporaryFile('w', self.rbufsize)
        self.wfile = TemporaryFile('w', self.wbufsize)
        # content
        self.rfile.write(content)
        # dup rfile to readonly - necessary but not sure why ???
        self.rfile.flush()
        tmpfile = self.rfile
        self.rfile = os.fdopen(os.dup(tmpfile.fileno()),'r')
        tmpfile.close()
        self.rfile.seek(0)
        # misc
        self.server = CGIServer(server_name,server_port)
        self.client_address = [headers.get('REMOTE-ADDR',''),'']
        self._client_address_string = headers.get('REMOTE-HOST','')
        # parse request
        self.parse_request()
        self.request = self

    def close(self):
        try: self.rfile.flush(); self.rfile.close() #; print "rfile closed"
        except: pass
        try: self.wfile.flush(); self.wfile.close() #; print "wfile closed"
        except: pass

    def address_string(self):
        return self._client_address_string

    def log_request(self, code='-', size='-'):
        self._log_message('log_request','"%s" %s %s',self.requestline,str(code),str(size))

    def log_message(self, format, *args):
        self._log_message('log_message',format,*args)

    def log_error(self, format, *args):
        self._log_message('log_error',format,*args)

    def _log_message(self, summary, format, *args):
        # message details
        format_details = format%args
        details = "%s - - [%s] %s\n" % (self.address_string(),
                                        self.log_date_time_string(),
                                        format_details)
        try:
            import zLOG
            if summary == 'log_request':
                severity = zLOG.INFO
            elif summary == 'log_message':
                severity = zLOG.INFO
            elif summary == 'log_error':
                severity = zLOG.ERROR
            else:
                severity = zLOG.PROBLEM

            zLOG.LOG('CGIHTTP',severity,summary,details)

            if severity > zLOG.INFO:
                raise CGIHTTPError, format_details

        except ImportError:
            import sys
            sys.stderr.write(details)

    def run_cgi(self):
        CGIHTTPRequestHandler.run_cgi(self)
        # dup wfile to readonly - necessary but not sure why ???
        self.wfile.flush()
        tmpfile = self.wfile
        self.wfile = os.fdopen(os.dup(tmpfile.fileno()),'r')
        tmpfile.close()
        self.wfile.seek(0)

# TemporaryFileWrapper used for debugging purposes only
class TemporaryFileWrapper:
    def __init__(self, mode='w+b', bufsize=-1, suffix=""):
        self.file = TemporaryFile(mode,bufsize,suffix)
        #from tempfile import mktemp
        #self.file = open(mktemp(suffix),mode,bufsize)

    def close(self):
        self.file.close()

    def __del__(self):
        try: self.close()
        except: pass

    def __getattr__(self, name):
        file = self.__dict__['file']
        a = getattr(file, name)
        if type(a) != type(0):
            setattr(self, name, a)
        return a

    def readline(self):
        file = self.__dict__['file']
        text = ''
        while 1:
            try:
                c = file.read(1)
            except IOError,EOFError:
                break
            else:
                pass
            text = text + c
            if c == '\n': break
        return text
